class CTerminateThreadException : public CException {
    public:
        CTerminateThreadException() : CException() { }
    };

class Queue {
    public:
       //----------------
       // Queue
       //----------------
       Queue(UINT limit)
	  {
	   handles[SemaphoreIndex] = ::CreateSemaphore(NULL,  // no security attributes
	                                               0,     // initial count
	                                               limit, // max count
					               NULL); // anonymous

	   handles[StopperIndex] = ::CreateEvent(NULL,        // no security attributes
				                 TRUE,        // manual reset
				                 FALSE,       // initially non-signaled
				                 NULL);       // anonymous
	   
	   ::InitializeCriticalSection(&lock);
	  } // Queue

       //----------------
       // ~Queue
       //----------------
       ~Queue()
	  {
	   ::CloseHandle(handles[SemaphoreIndex]);
	   ::CloseHandle(handles[StopperIndex]);
	   ::DeleteCriticalSection(&lock);
	  } // ~Queue

       //----------------
       // AddTail
       //----------------
       BOOL AddTail(LPVOID p)
	  { 
	   BOOL result;
	   ::EnterCriticalSection(&lock);
	   queue.AddTail(p);
	   result = ::ReleaseSemaphore(handles[SemaphoreIndex], 1, NULL);
	   if(!result)
	      { /* failed */
	       // caller can use ::GetLastError to determine what went wrong
	       queue.RemoveTail();
	      } /* failed */
	   ::LeaveCriticalSection(&lock);
	   return result;
	  } // AddTail

       //----------------
       // RemoveHead
       //----------------
       LPVOID RemoveHead()
	  { 
	   LPVOID result;
	   
	   switch(::WaitForMultipleObjects(2, handles, FALSE, INFINITE))
	      { /* decode */
	       case StopperIndex:   // shut down thread
		  throw new CTerminateThreadException;  // kill thread
		  return NULL;      // return keeps C compiler happy

	       case SemaphoreIndex: // semaphore
		  ::EnterCriticalSection(&lock);
		  result = queue.RemoveHead();
		  ::LeaveCriticalSection(&lock);
		  return result;

	       case WAIT_TIMEOUT: // not implemented
	       default:
		  ASSERT(FALSE); // impossible condition
		  return NULL;
	      } /* decode */
	  } // RemoveHead

       //----------------
       // shutdown
       //----------------
       void shutdown()
	  {
	   ::SetEvent(handles[StopperIndex]);
	  } // shutdown

    protected:
       enum {StopperIndex, SemaphoreIndex};
       HANDLE handles[2];
       CRITICAL_SECTION lock;
       CList<LPVOID, LPVOID> queue;
};
